#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
This module defines a list of payloads with various HTTP headers and their corresponding values.
The payloads are intended to test invalid, malformed, or unusual header values.
List of payloads:
- `payloads_keys`: A list of HTTP header and its value.
Examples of payloads:
- Headers with invalid characters or formats.
- Headers with extremely large values.
- Headers with conflicting values.
- Headers with non-standard or custom values.
- Headers with invalid date formats.
- Headers with invalid IP addresses.
- Headers with invalid JSON structures.
- Headers with invalid encryption keys.
- Headers with invalid cache control directives.
- Headers with invalid content types and encodings.
- Headers with invalid security policies.
This list is used for testing various edge cases and invalid inputs.
"""

payloads_keys = [  # {"xxxx":"缓"},
                   # {"缓":"缓"},
                   # {"X-Custom-Header-♥": "value"},
    {'xyz': '1'},
    {'(': '1'},
    {'/': '\\:\\'},
    {'"': '1'},
    {'\\': '1'},
    {'x-timer': 'x' * 8192},
    {'Cookie': 'sessionid=abc123\nmalformed'},
    {'Cookie': 'sessionid=abc123; Domain=example.com.evil.com'},
    {'Cookie': 'sessionid=abc123; Expires=InvalidDate'},
    {'Cookie': 'sessionid=' + 'a' * 8192},
    {'Cookie': 'sessionid=abc;123=value; malformed'},
    {'Cookie': 'sessionid=abc123\nX-Custom-Header: injected-value'},
    {'Cookie': 'sessionid=abc123; ; invalid_cookie='},
    {'Cookie': 'sessionid=abc 123'},
    {'Cookie': 'sessionid="abc123'},
    {'Cookie': 'sessionid=abc123; sessionid=def456'},
    {'User-Agent': 'Mozilla/5.0\nInvalid'},
    {'User-Agent': 'Mozilla/5.0\xC0'},
    {'User-Agent': 'Mozilla/5.0\n\nContent-Length: 0'},
    {'User-Agent': 'Mozilla/5.0\nX-Custom-Header: injected-value'},
    {'User-Agent': 'Mozilla/5.0 ' + 'a' * 8192},
    {'X-Custom-Header_1': 'Invalid\x00Header'},
    {'X-Custom-Header-@': 'value'},
    {'X-Custom-Header': '``'},
    {'Transfer-Encoding': 'chunked'},
    {'Transfer-Encoding': 'chunked; param=toto'},
    {'Transfer-Encoding': 'deflate'},
    {'Transfer-Encoding': 'compress'},
    {'Transfer-Encoding': 'gzip, chunked'},
    {'Transfer-Encoding': 'invalid-chunked'},
    {'Forwarded': 'xxxx'},
    {'Forwarded': 'host=xxx'},
    {'X-Timer': 'This_is_not_a_valid_value'},
    {'X-Timer': 'S1638483985.abcdef,VSinvalid,VEerror'},
    {'X-Timer': '5000'},
    {'X-Requested-With': 'SomeValue'},
    {'Authorization': 'Bearer InvalidToken'},
    {'Accept': 'toto'},
    {'Accept': 'application/json, application/xml'},
    {'Accept-Charset': 'utf-8, iso-8859-1'},
    {'Prefer': 'return=minimal, return=representation'},
    {'Prefer': 'unsupported-preference'},
    {'Prefer': 'return=minimal, return=representation'},
    {'Prefer': 'return=,wait=abc'},
    {'Prefer': 'wait=999999999'},
    {'Accept-Encoding': 'toto'},
    {'Accept-Encoding': 'gzip;q=1.0, identity;q=0.5, *;q=0'},
    {'Expect': '100-continue'},
    {'Expect': 'totoplop'},
    {'Etag': 'xxxx'},
    {'If-None-Match': 'etag123'},
    {'If-None-Match': '*', 'If-Match': 'toto'},
    {'If-None-Match': '<toto>'},
    {'If-Match': 'etag-value'},
    {'Max-Forwards': '0'},
    {'Max-Forwards': 'fooxxx'},
    {'TE': 'toto'},
    {'TE': 'chunked, gzip'},
    {'Connection': 'toto'},
    {'Content-Encoding': 'deflate'},
    {'Upgrade': 'foo/toto'},
    {'Proxy-Authorization': 'Basic dXNlcjpwYXNzd29yZA=='},
    {'Proxy-Authenticate': 'Basic realm=xxxx'},
    {'Via': '1.1 proxy.example.com'},
    {'DNT': '3'},
    {'Content-Disposition': 'invalid_value'},
    {'Warning': '199 - Invalid Warning'},
    {'Trailer': 'invalid_header'},
    {'Referer': 'xy'},
    {'Referer': 'xy, x'},
    {'Content-Length': '394616521189'},
    {'Content-Length': '-1'},
    {'Content-Type': 'application/invalid-type'},
    {'Retry-After': '-1'},
    {'Retry-After': 'fooxxx'},
    {'Retry-After': '1200'},
    {'X-RateLimit-Limit': '1000'},
    {'X-RateLimit-Remaining': '500'},
    {'X-RateLimit-Reset': '1581382800'},
    {'X-Content-Type-Options': 'fooxxx'},
    {'TE': 'teapot'},
    {'TE': 'fooxxx'},
    {'X-CF-APP-INSTANCE': 'xxx:1'},
    {'X-CF-APP-INSTANCE': 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa:1'},
    {'X_FORWARDED_PROTO': 'nohttps'},
    {'Cache-Control': 'no-cache'},
    {'If-Modified-Since': 'Wed, 21 Oct 2015 07:28:00 GMT'},
    {'If-Unmodified-Since': 'xxxxx'},
    {'Accept-Language': 'xxxxxx'},
    {'Accept-Language': 'en, fr, de'},
    {'Origin': 'https://unauthorized-origin.com'},
    {'From': 'user@example.com'},
    {'Pragme': 'toto'},
    {'Accept-Charset': 'Accept-Charset: utf-8, iso-8859-1;q=0.5'},
    {'DPR': '2.0'},
    {'Save-Data': 'on'},
    {'Sec-Fetch-Site': 'invalid-site'},
    {'Sec-Fetch-Mode': 'invalid-mode'},
    {'Sec-Fetch-Dest': 'invalid-dest'},
    {'Sec-Fetch-User': '?0'},
    {'Referrer-Policy': 'invalid-policy'},
    {'Permissions-Policy': 'invalid-permission'},
    {'Timing-Allow-Origin': '*'},
    {'Content-DPR': '1.0'},
    {'Early-Data': '1'},
    {'NEL': 'toto'},
    {'Reporting-Endpoints': 'toto'},
    {'Feature-Policy': "camera 'none'; microphone 'none'"},
    {'Clear-Site-Dat': 'cache'},
    {'Expect-CT': 'max-age=604800, enforce'},
    {'Expect-CT': 'max-age=-1'},
    {'Access-Control-Request-Method': 'POST'},
    {'Access-Control-Request-Method': 'xxxx'},
    {'Access-Control-Request-Headers': 'X-Custom-Header'},
    {'Upgrade-Insecure-Requests': '1'},
    {'Front-End-Https': 'toto'},
    {'Surrogate-Control': 'no-store'},
    {'X-Robots-Tag': 'noindex'},
    {'Service-Worker-Allowed': '/'},
    {'Cross-Origin-Embedder-Policy': 'require-corp'},
    {'Cross-Origin-Opener-Policy': 'same-origin'},
    {'Cross-Origin-Resource-Policy': 'same-origin'},
    {'Server-Timing': 'miss, db;dur=53, app;dur=47.2'},
    {'x-invoke-status': '888'},
    {'x-invoke-status': '404'},
    {'x-invoke-status': 'xxx'},
    {'x-invoke-status': '400'},
    {'x-invoke-status': '500'},
    {'x-invoke-status': '301'},
    {'x-invoke-status': '302'},
    {'x-invoke-error': 'xoxoxoxo'},
    {'x-invoke-ouput': 'xoxoxoxo'},
    {'x-invoke-path': 'xoxoxoxo'},
    {'x-invoke-query': 'xoxoxoxo'},
    {'Rsc': '1'},
    {'Rsc': 'xxx'},
    {'x-middleware-invoke': 'xoxoxoxo'},
    {'x-middleware-prefetch': '1'},
    {'Content-Encoding': 'toto'},
    {'Content-Type': 'text/html; charset=utf-16'},
    {'Content-Type': 'text/html; charset=utf-32'},
    {'Content-Type': 'text/html; charset=invalid-charset'},
    {'Content-Type': 'text/html; charset=invalid-charset',
     'Content-Encoding': 'toto'},
    {'Content-Type': 'application/json', 'Content-Encoding': 'gzip'},
    {'Content-Type': 'application/octet-stream',
     'Content-Encoding': 'deflate'},
    {'Content-Encoding': 'gzip, deflate'},
    {'Content-Language': 'xxxx'},
    {'Content-Location': 'xxxx'},
    {'Content-MD5': 'xxxx'},
    {'Content-Security-Policy': 'xxxx'},
    {'Content-Security-Policy': "default-src 'self'; img-src 'self' data:"
     },
    {'Content-Features': 'fooxxx=xxxx'},
    {'Content-Base': 'xxxx'},
    {'Content-Transfer-Encoding': 'xxxx'},
    {'Content-Style-Type': 'xxxx'},
    {'Content-Script-Type': 'xxxx'},
    {'Content-Label': 'xxxx'},
    {'Content-Warning': 'xxxx'},
    {'Content-Rate': 'xxxx'},
    {'Content-Digest': 'xxxx'},
    {'Range': 'bytes=nobytes'},
    {'Range': 'bytes=-10'},
    {'Range': 'bytes=200-300,100-150'},
    {'Range': 'bytes=0-50,100-150'},
    {'Range': 'Range: bytes=10000000-200000000'},
    {'Range': 'bytes=500-400'},
    {'Range': 'bytes=7000-'},
    {'Next-Router-State-Tree': '{"path":"/xxxx","params":{"id":"123"} }'
     },
    {'Next-Router-Prefetch': 'maybe'},
    {'Next-Url': '!@#$%^&*()'},
    {'CF-Connecting-IP': '999.999.999.999'},
    {'X-Accel-Redirect': '/invalid/path/to/resource'},
    {'X-Akamai-Edge-Cache': 'wrong-value'},
    {'CF-Visitor': '{"scheme":http}'},
    {'Surrogate-Control': 'invalid-directive'},
    {'X-Real-IP': 'abc.def.ghi.jkl'},
    {'X-Akamai-Request-ID': 'some-invalid-id'},
    {'CF-IPCountry': 'XYZ'},
    {'Fastly-Client-IP': '1234.567.89.0'},
    {'X-Accel-Expires': 'invalid-time'},
    {'next-router-state-tree': 'eyJyb3V0ZXMiOlt7InBhdGgiOiIvcHJvZmlsZSIsInF1ZXJ5Ijp7fX1dfQ=='
     },
    {'next-router-state-tree': 'eyJyb3V0ZXMiOlt7InBhdGgiOiJbadlyFormattedState==[/route]["bad"]'
     },
    {'next-render-mode': 'server'},
    {'next-data': 'eyJwYWdlIjoiZGFzaGJvYXJkIiwic2Vzc2lvbklkIjoiYWJjMTIzIn0='
     },
    {'next-data': '{"page": "dashboard", "sessionId": null}'},
    {'x-nextjs-page-name': '/xxxx'},
    {'x-nextjs-data': '{"id":1337,"name":"xxxx"}'},
    {'x-matched-path': '/incorrect/path'},
    {'x-nextjs-cache': 'invalid-value'},
    {'x-nextjs-nom': 'unexpected-value'},
    {'x-nextjs-revalidate': 'not-a-number'},
    {'x-nextjs-preview-data': '1'},
    {'x-nextjs-router-state-tree': 'corrupted-state'},
    {'x-nextjs-redirect': '://invalid-url'},
    {'x-nextjs-router-push': 'invalid-push-type'},
    {'x-middleware-next': '1'},
    {'x-nextjs-middleware-cache': 'invalid-cache-key'},
    {'next-js': '12.0.0'},
    {'x-nextjs-headers': 'Authorization,Invalid-Header-Name'},
    {'next-export-no-prerender': 'true'},
    {'next-ssg': 'false'},
    {'next-is-app-route': 'true'},
    {'next-response-type': 'xml'},
    {'x-execution-status': 'failure'},
    {'x-function-status': 'error'},
    {'x-service-status': 'degraded'},
    {'x-job-status': 'incompleted'},
    {'x-task-status': 'in-progress'},
    {'x-operation-status': 'unsuccess'},
    {'x-process-status': 'uncompleted'},
    {'x-task-retry': '20'},
    {'CF-Device-Type': 'xxxx'},
    {'CF-Device-Type': 'mobile'},
    {'True-Client-IP': '1234:5678:9abc:def0:1234:5678:9abc:def0:ghij'},
    {'CF-Ray': '6b8af3a3e8a9214f-XYZ123'},
    {'CF-Visitor': '{"scheme": "htp"}'},
    {'X-Forwarded-Proto': 'xxx'},
    {"x-forwarded-host": "ftp://xxxx"},
    {"x-forwarded-host": "mailto:xxxx"},
    {"x-forwarded-host": "file://xxxx"},
    {"x-forwarded-host": "toto.com"},
    {"x-forwarded-scheme": "nohttps"},
    {"X-Forwarded-Scheme": "nohttps", "X-Forwarded-Host": "example.co"},
    {"x-forwarded-proto": "nohttps"},
    {"X-Forwarded-Proto-Override": "nohttps"},
    {"x-forwarded-protocol": "nohttps"},
    {"x-forward-proto": "nohttps"},
    {"x-forwarded-proto": "31337"},
    {"X-Forwarded-Proto-orig": "nohttps"},
    {"X-Forwarded-Proto-Version": "nohttps"},
    {"X-Forwarded-SSL": "on"},
    {"X-Forwarded-SSL": "off"},
    {"X-Forwarded-Method": "POST"},
    {"X-Forwarded-Method": "HEAD"},
    {"X-Forwarded-Method": "PLOP"},
    {"X-Forwarded-Port": "31337"},
    {"X-Forwarded-Timeout": "120"},
    {"X-Forwarded-User": "xxx"},
    {"x-forwarded-host": "xxx"},
    {"X-Forwarded-URI": "xxx"},
    {"X-Forwarded-Access-Token": "xxx"},
    {"X-Forwarded-Session-ID": "xxx"},
    {"X-Forwarded-Client-Cert": "xxx"},
    {"x-forwarded-prefix": "xxx"},
    {"x-forwarded-url": "xxx"},
    {"Http-X-Forwarded-For-Ip": "xxx"},
    {"X-Forwarded-By": "xxx"},
    {"X-Forwarded-For-Original": "xxx"},
    {"X-Forwarded-For": "xxx"},
    {"X-Forwarded-Server": "xxx"},
    {"X-Forwarder-For": "xxx"},
    {"X-Forward-For": "xxx"},
    {'CF-Visitor-IP': '::1/128'},
    {'CF-Request-ID': '567890abcdef!@#'},
    {'CF-Pseudo-IPv4': '999.999.999.999'},
    {'CF-Visitor-Timezone': 'Mars/Phobos'},
    {'CF-Request-Started': '2023-99-99T99:99:99Z'},
    {'Fastly-Debug': 'not-a-boolean'},
    {'Surrogate-Control': 'max-age=abc123'},
    {'Fastly-FF': 'enable-feature-123'},
    {'Fastly-Soft-Purge': '12345'},
    {'Fastly-Cache-Control': 'no-store, max-age=not-a-number'},
    {'Fastly-Trace': 'on, version:five'},
    {'Surrogate-Key': '!@#$%^&*'},
    {'Fastly-Backend-Name': 'non_existent_backend'},
    {'Akamai-Cache-Control': 'max-age=not-a-number'},
    {'Akamai-Pragma-Client-IP': 'invalid-ip-format'},
    {'Akamai-Origin-Hop': '-1'},
    {'X-Akamai-Edge-Connect-Time': 'five'},
    {'X-Cache': 'INVALID_CACHE_STATUS'},
    {'X-Cache-Key': '/key/with spaces'},
    {'X-Akamai-Staging': 'invalid-value'},
    {'X-Akamai-Surrogate-Control': 'stale-while-revalidate=string'},
    {'X-True-Client-IP': '999.999.999.999'},
    {'X-Akamai-Edge-Result': 'error-unexpected-value'},
    {'X-Akamai-GEO': 'invalid-geo-code'},
    {'Akamai-Edge-IP': '::invalid:ip'},
    {'Akamai-Request-Time': 'timestamp-in-wrong-format'},
    {'Akamai-Request-Start': '2023-99-99T99:99:99Z'},
    {'Akamai-Request-UUID': 'incorrect-format-1234-!@#'},
    {'x-envoy-upstream-service-time': 'not-a-number'},
    {'x-envoy-retry-on': 'invalid-retry-condition'},
    {'x-envoy-max-retries': '-1'},
    {'x-envoy-upstream-healthchecked-cluster': 'non_existent_cluster'},
    {'x-envoy-expected-rq-timeout-ms': 'abc123'},
    {'x-envoy-internal': 'false'},
    {'x-envoy-original-path': '/invalid path with spaces'},
    {'x-envoy-force-trace': 'not-true-or-false'},
    {'x-envoy-decorator-operation': '!@#$%^&*'},
    {'x-envoy-downstream-service-cluster': 'wrong-format-cluster@123'},
    {'x-envoy-downstream-service-node': 'node#invalid_characters'},
    {'x-envoy-retry-grpc-on': 'invalid-grpc-status'},
    {'x-envoy-attempt-count': 'ten'},
    {'x-envoy-peer-metadata': '{"invalid-json}'},
    {'x-envoy-peer-metadata-id': '!@#$%^&*'},
    {'x-amz-website-redirect-location': '/new-path'},
    {'x-amz-meta-author': 'John Doe'},
    {'x-amz-server-side-encryption': 'AES256'},
    {'x-amz-server-side-encryption': 'AES256xxxx'},
    {'x-amz-storage-class': 'GLACIER'},
    {'x-amz-content-sha256': 'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'
     },
    {'x-amz-acl': 'public-read'},
    {'x-amz-tagging': 'project=website&env=prod'},
    {'x-amz-copy-source': '/source-bucket/source-object'},
    {'x-amz-metadata-directive': 'REPLACE'},
    {'x-amz-request-payer': 'requester'},
    {'x-amz-version-id': '3HL4kqCvBZEXAMPLEgj0Lp'},
    {'x-amz-request-id': '656c76696e6727732072657175657374'},
    {'x-amz-server-side-encryption-customer-algorithm': 'AES256'},
    {'x-amz-server-side-encryption-customer-algorithm': 'AES256xxxx'},
    {'x-amz-server-side-encryption-customer-key': 'base64-encoded-key'
     },
    {'x-amz-server-side-encryption-customer-key-MD5': 'base64-encoded-MD5'
     },
    {'x-amz-mfa': 'arn:aws:iam::123456789012:mfa/user 123456'},
    {'x-amz-content-length-range': '1024,1048576'},
    {'x-amz-copy-source-if-match': '6805f2cfc46c0f04559748bb039d69ae'},
    {'x-amz-copy-source-if-none-match': '6805f2cfc46c0f04559748bb039d69ae'
     },
    {'x-amz-notification-configuration': 'xxxxxxx'},
    {'x-amz-expiration': 'expiry-date="Thu, 01 Dec 2022 00:00:00 GMT", rule-id="rule-1"'
     },
    {'x-amz-replication-status': 'COMPLETED'},
    {'x-amz-delete-marker': 'true'},
    {'x-amz-restore': 'ongoing-request="true", expiry-date="Wed, 21 Dec 2022 00:00:00 GMT"'
     },
    {'x-amz-part-number': '2'},
    {'x-amz-abort-date': 'Wed, 01 Dec 2022 00:00:00 GMT'},
    {'x-amz-abort-rule-id': 'rule-2'},
    {'x-amz-server-side-encryption-aws-kms-key-id': 'arn:aws:kms:us-east-1:123456789012:key/abcd1234-56ef-78gh-90ij-klmnopqrstuv'
     },
    {'x-amz-bucket-object-lock-enabled': 'true'},
    {'x-amz-object-lock-mode': 'COMPLIANCE'},
    {'x-amz-object-lock-retain-until-date': '2023-12-01T00:00:00.000Z'
     },
    {'x-amz-security-token': 'invalid-token-1234'},
    {'x-amz-expected-bucket-owner': '000000000000'},
    {'x-amz-server-side-encryption-context': '{context-key:"missing-quotes"}'
     },
    {'x-amz-sns-message-type': 'InvalidType'},
    {'x-amz-lambda-invocation-type': 'InvalidInvocationType'},
    {'x-amz-lambda-log-type': 'InvalidLogType'},
    {'x-amz-api-gateway-stage': 'non-existent-stage'},
    {'x-amz-cf-id': 'invalid-cloudfront-id-1234'},
    {'x-amz-cloudfront-forwarded-for': '999.999.999.999'},
    {'x-amz-glacier-job-id': 'invalid-job-id-xyz'},
    {'x-amz-glacier-version': '9999-99-99'},
    {'x-amz-auto-multipart': 'invalid-value'},
    {'x-amz-auto-restore': 'invalid-value'},
    {'x-amz-log-result': 'invalid-value}'},
    {'x-amz-object-lock-legal-hold': 'ON'},
    {'x-ms-version': '0000-00-00'},
    {'x-ms-date': '2099-12-31T23:59:59Z'},
    {'x-ms-client-request-id': '!@#$%^&*()'},
    {'x-ms-meta-name-value': ''},
    {'x-ms-blob-type': 'unsupported_blob_type'},
    {'x-ms-copy-source': 'https://invalid-url'},
    {'x-ms-range': 'bytes=5000-4000'},
    {'x-ms-content-type': 'invalid/content-type'},
    {'x-ms-lease-id': 'invalid_lease_id'},
    {'x-goog-api-version': '999'},
    {'x-goog-request-params': 'invalid=params'},
    {'x-goog-user-project': 'invalid-project-id'},
    {'x-goog-acl': 'invalid-acl'},
    {'x-goog-meta-name': 'xxxx'},
    {'x-goog-encryption-key': 'invalid_encryption_key'},
    {'x-goog-if-match': 'nonexistent_etag'},
    {'x-goog-storage-class': 'invalid_storage_class'},
    {'x-goog-copy-source': 'https://invalid-source'},
    {'opc-client-info': 'invalid-client-info'},
    {'opc-request-id': '@!#$%^&*()'},
    {'opc-meta-name': ''},
    {'opc-retry-token': 'invalid_retry_token'},
    {'X-Squid-Error': 'ERR_ACCESS_DENIED 403'},
    {'X-Squid-Error': 'ERR_DNS_FAIL 502'},
    {'X-Squid-Error': 'ERR_CONNECT_FAIL 503'},
    {'X-Squid-Error': 'ERR_INVALID_REQ 400'},
    {'X-Squid-Error': 'ERR_READ_TIMEOUT 504'},
    {'X-Squid-Error': 'ERR_LIFETIME_EXP 504'},
    {'X-Squid-Error': 'ERR_FORWARDING_DENIED 403'},
    {'X-Squid-Error': 'ERR_ZERO_SIZE_OBJECT 502'},
    {'X-Squid-Error': 'ERR_CLIENT_ABORT 499'},
    {'X-Squid-Error': 'ERR_UNSUP_REQ 501'},
    {'X-Squid-Error': 'ERR_TOO_BIG 413'},
    {'X-Squid-Error': 'ERR_NO_RELAY 502'},
    {'X-Squid-Error': 'ERR_CACHE_MISS 404'},
    {'X-Squid-Error': 'ERR_SHUTDOWN 503}'},
    {'X-Backend-Server': 'xxxxx'},
    {'X-Unique-ID': '@!#$%^&*()INVALID_ID_TOO_LONG12345678901234567890'
     },
    {'X-Haproxy-Server-State': 'INVALID_STATE'},
    {'X-Haproxy-SSL-Client-Verify': '999'},
    {'X-Haproxy-SSL-Client-CN': '!@#$%^&*()INVALID_CN'},
    {'X-Haproxy-SSL-Client-SHA1': '123INVALID_SHA1_HASH_TOO_SHORT'},
    {'X-Haproxy-Server': 'invalid_haproxy_instance'},
    {'X-Haproxy-SSL-Client-Issuer': 'INVALID_ISSUER@!#'},
    {'X-Haproxy-SSL-Client-NotBefore': 'INVALID_DATE_FORMAT'},
    {'X-Haproxy-SSL-Client-NotAfter': 'INVALID_DATE_FORMAT'},
    {'X-Firebase-Locale': 'invalid-locale-code'},
    {'X-Firebase-AppCheck': 'invalid_app_check_token'},
    {'X-Client-Version': 'invalid_version_string'},
    {'X-Firebase-GMPID': 'invalid_gmp_id'},
    {'X-Firebase-Auth': 'invalid_auth_token'},
    {'X-Firebase-Auth-Debug': 'invalid_debug_flag'},
    {'X-Client-Data': 'invalid_client_data'},
    {'X-Firebase-ETag': 'invalid_etag_value'},
    {'X-Firebase-Request-Trace': 'invalid_trace_id'},
    {'X-Firebase-Request-Origin': 'invalid_origin'},
    {'X-Client-API-Platform': 'invalid_platform'},
    {'X-Firebase-Project-ID': 'invalid_project_id'},
    {'X-Firebase-Api-Key': 'invalid_api_key'},
    {'X-Firebase-Region': 'invalid_region'},
    {'X-Firebase-Client-ID': 'invalid_client_id'},
    {'X-Firebase-Client': 'invalid_client_version'},
    {'X-Firebase-Session-ID': 'invalid_session_id'},
    {'X-Firebase-Zero-Rating': 'invalid_zero_rating'},
    {'X-B3-TraceId': 'invalid_trace_id'},
    {'X-B3-SpanId': 'invalid_span_id'},
    {'X-B3-ParentSpanId': 'invalid_parent_span_id'},
    {'X-B3-Sampled': 'invalid_sample_value'},
    {'X-B3-Flags': 'invalid_flag'},
    {'X-B3-Debug': 'invalid_debug_value'},
    {'b3': 'invalid_combined_value'},
    {'X-WP-Nonce': 'invalid_nonce'},
    {'X-Pingback': 'invalid_pingback_url'},
    {'X-WP-Total': '-1'},
    {'X-WP-TotalPages': '-1'},
    {'X-Redirect-By': 'invalid_source'},
    {'X-WP-Auth': 'invalid'},
    {'X-WP-Admin-Bar': 'invalid'},
    {'X-WP-Comment-Status': 'unknown_status'},
    {'X-Cache-Status': 'MISS_ERROR'},
    {'X-WP-Debug': 'false'},
    {'X-WP-Login': 'failure'},
    {'X-Powered-By': 'invalid_cms'},
    {'X-WP-Post-Status': 'unknown'},
    {'X-Drupal-Cache': 'invalid_cache'},
    {'X-Drupal-Dynamic-Cache': 'unknown'},
    {'X-Frame-Options': 'invalid_origin'},
    {'X-Generator': 'invalid_version'},
    {'X-UA-Compatible': 'invalid_value'},
    {'X-Drupal-Route-Normalizer': '0'},
    {'X-Drupal-Fast-404': 'invalid'},
    {'X-Drupal-Fast-404': '1'},
    {'X-CSRF-Token': 'invalid_token'},
    {'X-Cache': 'INVALID'},
    {'X-Edge-Location': 'unknown_location'},
    {'X-Edge-IP': '999.999.999.999'},
    {'X-Edge-Cache-Status': 'BROKEN'},
    {'X-Cache-TTL': '-100'},
    {'X-Purge-Status': 'error'},
    {'X-Edge-Origin-Status': '999'},
    {'X-Edge-TTL': '-86400'},
    {'X-Edge-Error': 'INVALID_ERROR'},
    {'X-Azure-Ref': 'INVALID_REF'},
    {'X-Cache-Lookup': 'ERROR from nowhere'},
    {'X-MS-Request-ID': 'INVALID_ID'},
    {'X-MS-CG-Auth': 'INVALID_TOKEN'},
    {'X-Azure-CDN': 'Failure'},
    {'X-MS-Edge-Ref': 'INVALID_REF'},
    {'X-Cache-Lookup': 'ERROR from nowhere'},
    {'X-Goog-Trace-ID': 'INVALID_TRACE_ID'},
    {'X-Goog-Request-ID': 'INVALID_REQUEST_ID'},
    {'X-Goog-Stored-Content-Encoding': 'INVALID_ENCODING'},
    {'X-CDN77-Edge': 'unknown_location'},
    {'X-CDN77-Edge-IP': '999.999.999.999'},
    {'X-CDN77-Cache': 'BROKEN'},
    {'X-CDN77-TTL': '-100'},
    {'X-CDN77-Region': 'UNKNOWN_REGION'},
    {'X-CDN77-Geo': 'INVALID_GEO'},
    {'X-CDN77-RequestID': 'INVALID_REQUEST_ID'},
    {'X-Bunny-Cache': 'BROKEN'},
    {'X-Bunny-Edge': 'unknown_location'},
    {'X-Bunny-Edge-IP': '999.999.999.999'},
    {'X-Bunny-Storage': 'INVALID_STORAGE'},
    {'X-Bunny-TTL': '-100'},
    {'X-Bunny-Request-ID': 'INVALID_REQUEST_ID'},
    {'X-Bunny-Error': 'UNKNOWN_ERROR'},
    {'X-Tencent-Edge': 'unknown_location'},
    {'X-Tencent-Edge-IP': '999.999.999.999'},
    {'X-Tencent-Cache-Status': 'BROKEN'},
    {'X-Tencent-Request-ID': 'INVALID_REQUEST_ID'},
    {'X-Tencent-CDN-Error': 'UNKNOWN_ERROR'},
    {'X-Tencent-Region': 'UNKNOWN_REGION'},
    {'X-Tencent-Custom-ID': 'INVALID_CUSTOM_ID'},
    {'X-Ali-CDN-Edge': 'unknown_location'},
    {'X-Ali-CDN-Edge-IP': '999.999.999.999'},
    {'X-Ali-CDN-Request-ID': 'INVALID_REQUEST_ID'},
    {'X-Ali-CDN-TTL': '-86400'},
    {'X-Ali-CDN-Error': 'UNKNOWN_ERROR'},
    {'X-Ali-CDN-Region': 'INVALID_REGION'},
    {'X-Ali-CDN-Trace-ID': 'INVALID_TRACE_ID'},
    {'X-PJAX-Container': '!invalidSelector'},
    {'X-PJAX-Container': '123InvalidSelector'},
    {'Turbo-Frame': '!invalidFrameId'},
    {'Turbo-Frame': '123InvalidFrameId'},
    {'X-PJAX': 'invalidValue'},
    {'X-PJAX': '!'},
    {'X-PJAX': '12345'},
    {'Turbo-Drive': '0'},
    {'Turbo-Drive': 'undefined'},
    {'Turbo-Drive': '!@#$%'},
    {'x-ms-lease-status': 'invalid_status'},
    {'x-ms-copy-status': 'invalid_copy_status'},
    {'x-ms-routing-name': 'invalid_routing_slot'},
    {'x-ms-client-principal-name': 'INVALID_USER'},
    {'X-Incap-Request-ID': 'INVALID_REQUEST_ID'},
    {'X-Incap-Signature': 'INVALID_SIGNATURE'},
    {'X-Incap-Session-Id': 'invalid_session_id'},
    {'X-Imperva-Region': 'geoerror'},
    {'X-Connection-Options': 'invalid_option'},
    {'X-Content-Type-Options': 'nosniff'},
    {'X-Frame-Options': 'DENY'},
    {'X-Incap-Cache-Control': 'no-store'},
    {'X-Imperva-Header': 'InvalidHeader'},
    {'X-Incap-Request-Time': 'InvalidTimestamp'},
    {'X-Imperva-Error-Code': '403'},
    {'X-Imperva-Error-Message': 'ServiceUnavailable'},
    {'X-Imperva-Error-Description': 'TimeoutError'},
    {'X-Imperva-Error-Info': '403 Forbidden - Custom Block'},
    {'X-Incap-Response-Cache-Control': 'no-cache'},
    {'X-Imperva-TrustLevel': 'low'},
    {'X-Imperva-Caching': 'stale'},
    {'X-Imperva-WAF': 'blocked'},
    {'X-Incap-Response-Status': '403 Forbidden'},
    {'X-Incap-Response-Error': 'RequestTimeout'},
    {'X-Imperva-Access-Control': 'AccessDenied'},
    {'X-Iinfo': 'INVALID_INFO_FORMAT'},
    {'X-CDN': 'INVALID_CDN'},
    {'X-Imperva-ID': 'INVALID_IMPERVA_ID'},
    {'X-Block-ID': 'INVALID_BLOCK_ID'},
    {'X-Incap-Client-IP': 'INVALID_CLIENT_IP'},
    {'X-WAF-Events': 'INVALID_EVENT'},
    {'X-Weebly-Request-ID': 'xxxxxxx'},
    {'X-Weebly-Site-ID': 'xxxxxx'},
    {'X-Contentful-User-Agent': 'invalid-sdk 1.0.0'},
    {'X-Contentful-Request-Id': 'INVALID_REQUEST_ID'},
    {'X-Contentful-Version': 'INVALID_VERSION'},
    {'X-Contentful-Content-Type': 'nonexistentContentType'},
    {'X-Contentful-RateLimit-Reset': '-1'},
    {'X-Contentful-RateLimit-Second-Limit': '0'},
    {'X-Contentful-RateLimit-Second-Remaining': '-10'},
    {'X-Contentful-Api-Version': '999'},
    {'X-TYPO3-Parsetime': 'INVALID_TIME'},
    {'X-TYPO3-Parsetime': '-1'},
    {'X-TYPO3-Cache': 'INVALID_CACHE_STATUS'},
    {'X-TYPO3-Domain': 'invalid.domain.com'},
    {'X-TYPO3-Sitename': 'InvalidSiteName'},
    {'X-TYPO3-PageID': '99999'},
    {'X-TYPO3-Language': 'invalid_language_code'},
    {'X-Wix-Request-ID': 'INVALID_REQUEST_ID'},
    {'X-Seen-By': 'unknown_server'},
    {'X-Wix-Published-Version': 'INVALID_VERSION'},
    {'X-Wix-Server-Artifact': 'INVALID_ARTIFACT'},
    {'X-Wix-Renderer-Server': 'INVALID_RENDERER'},
    {'X-Wix-Cache-Status': 'INVALID_STATUS'},
    {'X-Wix-Correlation-Id': 'INVALID_CORRELATION_ID'},
    {'X-Wix-Shard-ID': 'INVALID_SHARD'},
    {'X-JspTaglib': 'nonexistent_taglib'},
    {'X-Servlet-Engine': 'UnknownEngine/0.0'},
    {'X-Java-Update': 'Java/invalid_version'},
    {'X-Java-Version': 'UnknownVersion'},
    {'Server': 'UnknownServer/0.0'},
    {'X-Matomo-Device-ID': 'invalid_device_id'},
    {'X-Matomo-Session-ID': 'invalid_session_id'},
    {'X-Matomo-Token': 'invalid_token'},
    {'X-Matomo-Site-ID': '9999'},
    {'X-Matomo-Visitor-ID': 'invalid_visitor_id'},
    {'X-Matomo-Do-Not-Track': '2'},
    {'X-Matomo-Correlation-ID': 'invalid_correlation_id'},
    {'X-JspTaglib': 'nonexistent_taglib'},
    {'X-Servlet-Engine': 'UnknownEngine/0.0'},
    {'X-Java-Update': 'Java/invalid_version'},
    {'X-Java-Version': 'UnknownVersion'},
    {'Server': 'UnknownServer/0.0'},
    {'X-Matomo-Device-ID': 'invalid_device_id'},
    {'X-Matomo-Session-ID': 'invalid_session_id'},
    {'X-Matomo-Token': 'invalid_token'},
    {'X-Matomo-Site-ID': '9999'},
    {'X-Matomo-Visitor-ID': 'invalid_visitor_id'},
    {'X-Matomo-Do-Not-Track': '2'},
    {'X-Matomo-Correlation-ID': 'invalid_correlation_id'},
    {'Strict-Transport-Security': 'invalid-value'},
    {'X-XSS-Protection': 'invalid-value'},
    {'Access-Control-Allow-Origin': 'invalid-origin'},
    {'Accept-CH': 'invalid-header'},
    {'X-Permitted-Cross-Domain-Policies': 'invalid-policy'},
    {'Access-Control-Allow-Credentials': 'invalid-value'},
    {'Feature-Policy': 'invalid-feature'},
    {'X-Download-Options': 'invalid-value'},
    {'X-Content-Duration': 'invalid-duration'},
    {'X-Apache-Error': 'invalid-error-code'},
    {'X-Tengine-Error': 'invalid-error-code'},
    {'X-Powered-By-Tengine': 'invalid-version'},
    {'Tengine-Cache': 'invalid-status'},
    {'X-Tengine-Request-ID': 'invalid-id'},
    {'X-Tengine-Forwarded-For': 'invalid-ip'},
    {'X-Tengine-Node': 'invalid-node'},
    {'Tengine-Upstream-Status': 'invalid-status'},
    {'X-Squarespace-Cache-Status': 'invalid-status'},
    {'X-Squarespace-Server': 'invalid-server'},
    {'X-Squarespace-Request-ID': 'invalid-id'},
    {'X-Squarespace-Revision': 'invalid-revision'},
    {'Squarespace-CDN': 'invalid-cdn'},
    {'X-Squarespace-Template': 'invalid-template'},
    {'x-nuxt-cache': 'invalid'},
    {'x-nuxt-middleware': 'error'},
    {'x-nuxt-error': 'error'},
    {'X-Nuxt-Dev': 'invalid-value'},
    {'X-Nuxt-SSR': 'not-a-boolean'},
    {'X-Nuxt-Static': 'not-a-static-value'},
    {'X-Nuxt-Request-ID': 'invalid-id'},
    {'X-Nuxt-Route': 'invalid-route'},
    {'X-Gatsby-Source': 'invalid-source'},
    {'X-Gatsby-Cache-Control': 'invalid-value'},
    {'X-Gatsby-Redirect': 'invalid-redirect'},
    {'X-Gatsby-Static': 'not-static'},
    {'X-Gatsby-Request-ID': 'invalid-request'},
    {'X-Gatsby-Feature-Flag': 'unsupported-feature'},
    {'X-Sapper-Environment': 'invalid-environment'},
    {'X-SvelteKit-Mode': 'unsupported-mode'},
    {'X-Sapper-Error': 'custom-error'},
    {'X-SvelteKit-Data': 'invalid-data'},
    {'X-Sapper-Client': 'invalid-client'},
    {'X-SvelteKit-Feature': 'unsupported-feature'},
    {'X-Remix-Environment': 'invalid-environment'},
    {'X-Remix-Cache-Control': 'invalid-value'},
    {'X-Remix-Request-ID': 'invalid-id'},
    {'X-Remix-Error': 'custom-error'},
    {'X-Remix-Data': 'invalid-data'},
    {'X-Remix-Feature': 'unsupported-feature'},
    {'X-Laravel-Session': 'invalid-session-id'},
    {'X-Laravel-Request-ID': 'invalid-request-id'},
    {'X-Laravel-Cache': 'invalid-cache-control'},
    {'X-Laravel-Error': 'custom-error'},
    {'X-Laravel-Environment': 'invalid-environment'},
    {'X-React-Component': 'InvalidComponent'},
    {'X-React-StrictMode': 'true'},
    {'X-Angular-Version': '0.0.0'},
    {'X-Angular-SSR': 'not-a-boolean'},
    {'X-Angular-Cache': 'invalid-cache-header'},
    {'X-Vue-Instance': 'invalid-instance'},
    {'X-Vue-SSR': 'not-a-boolean'},
    {'X-Vue-Devtools': 'true'},
    {'X-Vue-Request-ID': 'invalid-id'},
    {'X-Django': 'InvalidDjango'},
    {'X-Django-Version': '0.0.0'},
    {'X-Django-Settings': 'invalid-settings'},
    {'X-Django-Request-ID': 'invalid-request-id'},
    {'X-Rails': 'InvalidRails'},
    {'X-Rails-Version': '0.0.0'},
    {'X-Rails-Request-ID': 'invalid-request-id'},
    {'X-Rails-Cache': 'invalid-cache'},
    {'X-Powered-By': 'InvalidServer'},
    {'X-Express': 'InvalidExpress'},
    {'X-Express-Version': '0.0.0'},
    {'X-Express-Request-ID': 'invalid-request-id'},
    {'X-Express-Rate-Limit': 'invalid-rate'},
    {'X-AspNet-Version': '0.0.0'},
    {'X-AspNetMvc-Version': '0.0.0'},
    {'X-AspNet-Request-ID': 'invalid-request-id'},
    {'X-AspNet-Cache': 'invalid-cache'},
    {'X-Spring-Boot-Version': '0.0.0'},
    {'X-Spring-Version': '0.0.0'},
    {'X-Spring-Request-ID': 'invalid-request-id'},
    {'X-Spring-Cache': 'invalid-cache'},
    {'X-Flask-Version': '0.0.0'},
    {'X-Flask-Env': 'invalid-env'},
    {'X-Flask-Request-ID': 'invalid-request-id'},
    {'X-Flask-Cache': 'invalid-cache'},
    {'X-Laravel-Version': '0.0.0'},
    {'X-Laravel-Env': 'invalid-env'},
    {'X-Nextjs-Version': '0.0.0'},
    {'X-Nextjs-SSR': 'not-a-boolean'},
    {'X-Nextjs-Cache': 'invalid-cache'},
    {'X-Nextjs-Request-ID': 'invalid-request-id'},
    {'X-Nuxt-Version': '0.0.0'},
    {'X-Svelte-Version': '0.0.0'},
    {'X-Svelte-Dev': 'not-a-boolean'},
    {'X-Svelte-Request-ID': 'invalid-request-id'},
    {'X-Svelte-Cache': 'invalid-cache'},
    {'X-Symfony-Version': '0.0.0'},
    {'X-Symfony-Env': 'invalid-env'},
    {'X-Symfony-Request-ID': 'invalid-request-id'},
    {'X-Symfony-Cache': 'invalid-cache'},
    {'X-Gatsby-Version': '0.0.0'},
    {'X-Gatsby-SSR': 'not-a-boolean'},
    {'X-Gatsby-Cache': 'invalid-cache'},
    {'X-React-DevTools': 'disabled'},
    {'X-React-Profiler': 'not-a-boolean'},
    {'X-Angular-TrackBy': 'invalid-track'},
    {'X-Vue-Strict': 'true'},
    {'X-Vue-Error': 'some-error'},
    {'X-Django-Allowed-Hosts': 'invalid.host.com'},
    {'X-Django-Cookies': 'invalid-cookie'},
    {'X-Rails-Error': 'some-error'},
    {'X-Rails-Force-SSL': 'true'},
    {'X-Express-Error': 'some-error'},
    {'X-Express-Body-Parser': 'invalid-option'},
    {'X-AspNet-Secure': 'false'},
    {'X-AspNet-Session-Timeout': 'not-a-number'},
    {'X-Spring-Active-Profile': 'invalid-profile'},
    {'X-Flask-Static-Path': 'invalid-path'},
    {'X-Laravel-Migration-Status': 'invalid-status'},
    {'X-Nextjs-Redirect': 'invalid-url'},
    {'X-Nuxt-Route-Middleware': 'invalid-middleware'},
    {'X-Svelte-Compile-Options': 'invalid-options'},
    {'X-Symfony-Cache-Key': 'invalid-key'},
    {'X-Gatsby-Image-Processing': 'invalid-processing'},
    {'X-WP-Engine-No-Cache': 'invalid-no-cache'},
    {'X-Pass-Why': 'invalid-reason'},
    {'X-Total-Count': '-1'},
    {'X-Content-Range': '-1'},
    {'X-Total-Count': '100'},
    {'X-Content-Range': '100'},
    {'X-Strapi-Cache': 'toto'},
    {'X-Strapi-Cache': 'true'},
    {'X-Strapi-Warn': 'true'},
    {'X-Strapi-Warn': 'toto'},
    {'X-Esi': '2'},
    {'X-Esi': 'xxxx'},
    {'X-ESI-Enabled': 'invalid'},
    {'X-ESI-Enabled': 'false'},
    {'Public-Key-Pins': 'pin-sha256="invalidhash=="; max-age=5184000'},
    {'Public-Key-Pins': 'pin-sha256="primaryhash=="; max-age=5184000'},
    {'Public-Key-Pins': 'pin-sha256="primaryhash=="; pin-sha256="backuphash=="; max-age=-1'
     },
    {'Public-Key-Pins': 'pin-sha256="primaryhash=="; pin-sha256="backuphash=="; max-age=5184000; includeSubDomains=true'
     },
    {'Public-Key-Pins': 'pin-sha256="primaryhash=="; pin-sha256="backuphash=="; max-age=5184000; report-uri="http://"'
     },
    {'If-Match': 'xoxoxox', 'Accept-Encoding': 'gzip'},
    {'If-Match': 'xoxoxox', 'Accept-Encoding': 'br'},
    {'P3P': 'invalid-policy-ref'},
    {'X-Runtime': 'notatime'},
    {'X-Accel-Buffering': 'invalid'},
    {'X-Content-Transfer-Encoding': 'invalid-transfer-encoding'},
    {'X-Content-Type-Option': 'invalid'},
    {'X-Node-Version': '0.0.0'},
    {'X-Cloud-Trace-Context': 'invalid'},
    {'X-Request-Context': 'invalid'},
    {'X-Content-Security-Policy': 'invalid'},
    {'X-Server-Time': 'notime'},
    {'X-DNS-Prefetch-Control': 'off'},
    {'X-DNS-Prefetch-Control': 'nok'},
    {'X-Error-Code': 'nok'},
    {'X-TLS-Protocol': 'TLSv1.0'},
    {'X-Encoding': 'nok'},
    {'x-next-debug-logging': '1'},
    {'x-next-debug-logging': '100'},
    {'x-next-debug-logging': 'enable_enable'},
    {'Set-Cookie': 'sessionid=abc123; HttpOnly; Secure; Max-Age=-100'},
    {'Access-Control-Allow-Methods': 'CONNECT, DELETE, INVALID'},
    {'Access-Control-Allow-Headers': 'Authorization,Invalid-Header'},
    {'Access-Control-Expose-Headers': 'Authorization, X-Custom-Header, invalid-header'},
    {'X-RSC-Status': 'error'},
    {'X-RSC-Cache': 'miss'},
    {'X-RSC-Content-Type': 'application/json; charset=utf-8'},
    {'X-RSC-Mode': 'streaming'},
    {'X-RSC-Exception': 'true'},
    {'X-RSC-Expires': '0'},
    {'X-RSC-Renderer': 'server'},
    {'X-RSC-Data': 'invalid-data'},
    {'X-RSC-Request-ID': '12345abcde'},
    {'X-RSC-Response-Time': 'invalid-value'},
    {'Location': 'ftp://xxxx'},
    {'Location': 'mailto:xxxx'},
    {'Location': 'file://xxxx'},
    {'Content-Type': 'application/x-ftp'},
    {'Location': 'javascript:alert("XSS")'},
    {'Location': 'data:text/html;base64,PGh0bWw+PHNjcmlwdD5hbGVydCgnWFNTJyk8L3NjcmlwdD48L2h0bWw+'},
    {"X-Now-Route-Matches": "1"},
    {"X-Now-Route-Matches": "toto"},
    {"X-Middleware-Rewrite": "xxxxx"},
    {"X-Middleware-Rewrite": "/new-path"},
    {"X-Nextjs-Page": "/xxxx"},
    {"X-Nuxt-Mode": "toto"},
    {"X-Sveltekit-SSR": "toto"},
    {"X-Sveltekit-Router": "/xxxx"},
    {"X-Remix-Data": "toto"},
    {"X-Remix-Loader": "toto"},
    {"X-Astro-Static-Asset": "toto"},
    {"X-Astro-Redirect": "/toto"},
    {"X-Blitz-Session": "user123-session-token"},
    {"X-Blitz-Context": "toto"},
    {"X-Blitz-Route": "/xxxx"},
    {"X-Gatsby-Prerende": "toto"},
    {"X-Feature-Flag": "new-dashboard"},
    {"X-Environment": "toto"},
    {"X-User-Role": "toto"},
    {"X-Client-ID": "abc123"},
    {"X-Api-Key": "xyz987654321"},
    {"X-User-Language": "fr"},
    {"X-User-Timezone": "Europe/Paris/toto"},
    {"X-User-Preferences": "toto"},
    {"X-Max-Results": "-100"},
    {"X-Timeout": "-30"},
    {"X-Service-ID": "inventory-servicetoto"},
    {"X-Service-Caller": "orders-servicetto"},
    {"Region": "coucou"},
    {'X-ORDS-Media-Type': 'invalid/type'},  
    {'X-APEX-STATUS-CODE': '999'},  
    {'WL-Proxy-Client-IP': '0.0.0.0'},  
    {'Oracle-ECID': 'INVALID-ECID'},  
    {'Oracle-DMS-ECID': 'ERROR-ECID'},  
    {'X-DB-Content-Length': '-1'},
    {'X-DB-Client-Info': 'corrupted-client'},  
    {'X-Oracle-Cache-Control': 'no-store'},  
    {'X-Oracle-Session': 'INVALIDSESSION'},  
    {'X-Oracle-Charset': 'UNKNOWN-CHARSET'},
    {'X-Oracle-Diagnostic-ID': 'ERROR-DIAG'},  
    {'X-Oracle-Request-ID': 'MALFORMED-REQ-ID'},  
    {'X-Oracle-Auth': 'BAD-CREDENTIALS'},  
    {'SOAPAction': 'InvalidSOAPAction'},  
    {'X-ORACLE-FMW-TRACING': 'DISABLED'},  
    {'X-ORACLE-DB-SQL': 'DROP TABLE users;'},  
    {'X-Oracle-Token': 'EXPIREDTOKEN'},
    {'X-Oracle-Retry-After': '-10'},  
    {'X-Oracle-MaxRows': '99999999'},  
    {'X-Oracle-Fusion-Request': 'INVALID'},  
    {'X-ORACLE-SECURITY': 'WEAK-HASH'},  
    {'X-ORACLE-ERROR-MODE': 'DEBUG'},  
    {'X-ORACLE-SESSION-EXPIRY': '0'},  
    {'X-Oracle-API-Version': '999.999'},  
    {'X-Oracle-Instance-ID': 'NONEXISTENT'},  
    {'X-ORACLE-TRANSACTION-ID': 'ERROR-TRANSACTION'},  
    {'X-Oracle-LB-Status': 'FAIL'},  
    {'X-Oracle-DB-Mode': 'INVALID-MODE'},  
    {'X-Oracle-CORS-Policy': 'BAD-CORS'},  
    {'X-Oracle-Connection': 'TIMEOUT'},  
    {'X-Oracle-Resource-ID': 'MISSING'},  
    {'X-Oracle-Region': 'UNKNOWN'},  
    {'X-Oracle-Query-Limit': '-100'},  
    {'X-Oracle-Cache-TTL': '-1'},  
    {'X-Oracle-Rate-Limit': '999999'},  
    {'X-Oracle-Data-Format': 'UNSUPPORTED'},  
    {'X-Oracle-Content-Encoding': 'INVALID'},  
    {'X-Oracle-Audit-Log': 'DISABLED'},  
    {'X-Oracle-Request-Timeout': '999999'},  
    {'X-Oracle-Validation-Status': 'FAIL'},  
    {'X-Oracle-Metadata': 'CORRUPTED'},  
    {'X-Oracle-Deployment-ID': 'NOT-FOUND'},  
    {'X-Oracle-Integrity-Check': 'FAILED'},  
    {'X-Oracle-Security-Policy': 'WEAK'},  
    {'X-Oracle-Failover': 'DISABLED'},  
    {'X-Oracle-Cluster-ID': 'INVALID'},  
    {'X-Oracle-Query-Cache': 'OFF'},  
    {'X-Oracle-Backup-ID': 'MISSING'},  
    {'X-Oracle-Cloud-Status': 'ERROR'},  
    {'X-Oracle-Service-ID': 'UNAVAILABLE'},  
    {'X-Oracle-Locale': 'UNKNOWN-LOCALE'},
    {'X-CDN-Request-ID': 'INVALID-ID'},
    {'X-JSD-Cache': 'DISABLED'},
    {'X-JSD-File-Version': '999999'},
    {'X-JSD-File-Integrity': 'FAILED'},
    {'X-JSD-Status': 'ERROR'},
    {'X-JSD-Proxy': 'BYPASS'},
    {'X-JSD-Rate-Limit': 'EXCEEDED'},
    {'X-JSD-Blocked': 'true'},
    {'X-JSD-Origin': 'INVALID'},
    {'Link': 'xxxxxxx'},
    {'Refresh': 'xxxxxxx'},
    {'Status': 'xxxxxxx'},
    {'WWW-Authenticate': 'xxxxxxx'},
    {'X-WebKit-CSP': 'xxxxxxx'},
    {'WP-NGINX-CONFIG': 'xxxxxxx'},
    {'SCRIPT-FILENAME': 'xxxxxxx'},
    {'RAPID-CACHE-ALLOWED': 'xxxxxxx'},
    {'REQUEST-METHOD': 'xxxxxxx'},
    {'REQUEST-URI': 'xxxxxxx'},
    {'REMOTE-ADDR': 'xxxxxxx'},
    {'SERVER-SOFTWARE': 'xxxxxxx'},
    {'SERVER-PROTOCOL': 'xxxxxxx'},
    {'SERVER-PORT': 'xxxxxxx'},
    {'SERVER-ADDR': 'xxxxxxx'},
    {'EDGE-CACHE-ENGINE': 'xxxxxxx'},
    {'X-CACHE-ENABLED': 'xxxxxxx'},
    {'X-VARNISH': 'xxxxxxx'},
    {'WPINC': 'xxxxxxx'},
    {'RAPID-CACHE-PLUGIN-FILE': 'xxxxxxx'},
    {'RAPID-CACHE-AC-FILE-VERSION': 'xxxxxxx'},
    {'WP-DEBUG': 'xxxxxxx'},
    {'RAPID-CACHE-PRO': 'xxxxxxx'},
    {'RAPID-CACHE-ENABLE': 'xxxxxxx'},
    {'RAPID-CACHE-DEBUGGING-ENABLE': 'xxxxxxx'},
    {'RAPID-CACHE-ALLOW-CLIENT-SIDE-CACHE': 'xxxxxxx'},
    {'RAPID-CACHE-GET-REQUESTS': 'xxxxxxx'},
    {'RAPID-CACHE-IGNORE-GET-REQUEST-VARS': 'xxxxxxx'},
    {'RAPID-CACHE-CACHE-404-REQUESTS': 'xxxxxxx'},
    {'RAPID-CACHE-CACHE-NONCE-VALUES': 'xxxxxxx'},
    {'RAPID-CACHE-CACHE-NONCE-VALUES-WHEN-LOGGED-IN': 'xxxxxxx'},
    {'RAPID-CACHE-FEEDS-ENABLE': 'xxxxxxx'},
    {'RAPID-CACHE-DIR': 'xxxxxxx'},
    {'RAPID-CACHE-MAX-AGE': 'xxxxxxx'},
    {'RAPID-CACHE-EXCLUDE-HOSTS': 'xxxxxxx'},
    {'RAPID-CACHE-EXCLUDE-URIS': 'xxxxxxx'},
    {'RAPID-CACHE-EXCLUDE-CLIENT-SIDE-URIS': 'xxxxxxx'},
    {'RAPID-CACHE-EXCLUDE-REFS': 'xxxxxxx'},
    {'RAPID-CACHE-EXCLUDE-AGENTS': 'xxxxxxx'},
    {'RAPID-CACHE-404-CACHE-FILENAME': 'xxxxxxx'},
    {'XMLRPC-REQUEST': 'xxxxxxx'},
    {'REST-REQUEST': 'xxxxxxx'},
    {'X-LiteSpeed-Debug': 'xxxxxxx'},
    {'x-litespeed-cache': 'xxxxxxx'},
    {'x-lsadc-cache': 'xxxxxxx'},
    {'x-qc-cache': 'xxxxxxx'},
    {'LSCACHE-VARY-VALUE': 'xxxxxxx'},
    {'X-LSCACHE-VARY-VALUE': 'xxxxxxx'},
    {'X-LiteSpeed-Vary': 'xxxxxxx'},
    {'W3TCALWAYSCACHED': 'xxxxxxx'},
    {'X-APPLICATION': 'xxxxxxx'},
    {'X-GT-LANG': 'xxxxxxx'},
    {'CLOUDFRONT-IS-MOBILE-VIEWER': 'xxxxxxx'},
    {'Content-Type': 'application/yaml'},
    {'X-Host': 'xxxxxxx'},
    {'Handle': 'xxxxxxx'},
    {'H0st': 'xxxxxxx'},
    {'X-Original-Url': 'xxxxxxx'},
    {'X-Original-Host': 'xxxxxxx'},
    {'Fastly-Host': 'xxxxxxx'},
    {'Api-Version': 'xxxxxxx'},
    {'AcunetiX-Header': 'xxxxxxx'},
    {'X-Original-Url': 'plopiplop', 'X-HTTP-Method-Override': 'POST'},
    {'X-Back-End-Server': 'plopiplop'},
    {'X-Request-ID': 'plopiplop'},
    {'X-Edge-Request-ID': 'plopiplop'},
    {'X-CDN-Tier': 'plopiplop'},
    {'CDN-Status':'404'},
    {'CDN-Status': 'plopiplop'},
    {'CDN-RequestId': 'plopiplop'},
    {'X-BunnyCDN-Cache': 'plopiplop'},
    {'X-BunnyCDN-Tier': 'plopiplop'},
    {'X-BunnyCDN-Edge': 'plopiplop'},
    {'X-BunnyCDN-Request-ID': 'plopiplop'},
    {'X-BunnyCDN-Error': 'plopiplop'},
    {'X-BunnyCDN-Forwarded-For': 'plopiplop'},
    {'X-BunnyCDN-Region': 'plopiplop'},
    {'CDN-RequestPullCode': 'plopiplop'},
    {'CDN-PullZone': 'plopiplop'},
    {'CDN-RequestCountryCode': 'plopiplop'},
    {'CDN-ProxyVer': 'plopiplop'},
    {'CDN-Uid': 'plopiplop'},
    {'Depth': '1'},
    {'Depth': 'ploptoto'},
    {'X-Requested-With': 'XMLHttpRequest'},
    {'X-Response-Format': 'json'},
    {'X-Response-Format': 'ploptoto'},
    {'X-API-Version': '2'},
    {'X-API-Version': 'ploptoto'},
    {'X-PJAX': 'true'},
    {'X-GitHub-Media-Type': 'github.v3'},
    {'X-GitHub-Media-Type': 'ploptoto'},
    {'X-Format': 'json'},
    {'X-Format': 'ploptoto'},
    {'X-Requested-Format': 'json'},
    {'X-Requested-Format': 'ploptoto'},
    {'X-Http-Accept': 'application/json'},
    {'X-Client-Accepts': 'application/json'},
    {'X-Response-Type': 'json'},
    {'X-Force-JSON': 'true'},
    {'X-Shopify-Access-Token': 'your_token'},
    {'X-Firebase-Locale': 'en-US'},
    {'X-Twitter-Client-Version': 'latest'},
    {'X-Amz-Target': 'SomeAWSService.Action'},
    {'X-Nuxt-Middleware': 'true'},
    {'X-Nextjs-Data': 'true'},
    {'X-Express-Json': 'true'},
    {'X-Laravel-Ajax': 'true'},
    {'X-Debug-JSON': 'true'},
    {'X-Override-Accept': 'application/json'},
    {'X-Api-Response': 'json'},
    {'X-Http-Accept': 'ploptoto'},
    {'X-Client-Accepts': 'ploptoto'},
    {'X-Response-Type': 'ploptoto'},
    {'X-Force-JSON': 'ploptoto'},
    {'X-Shopify-Access-Token': 'ploptoto'},
    {'X-Firebase-Locale': 'ploptoto'},
    {'X-Twitter-Client-Version': 'ploptoto'},
    {'X-Amz-Target': 'ploptoto'},
    {'X-Nuxt-Middleware': 'ploptoto'},
    {'X-Nextjs-Data': 'ploptoto'},
    {'X-Express-Json': 'ploptoto'},
    {'X-Laravel-Ajax': 'ploptoto'},
    {'X-Debug-JSON': 'ploptoto'},
    {'X-Override-Accept': 'ploptoto'},
    {'X-Api-Response': 'ploptoto'},
    {'x'*256: 'plop'},
    {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) (This is a comment'},
    {'Authorization': 'Bearer "my token with "quotes" inside"'},
    {'Upgrade': 'HTTP/2.0, SHTTP/1.3, IRC/6.9, RTA/x11'},
    {'Upgrade': 'websocket'},
    {'Upgrade': 'h2c'},
    {'Upgrade': 'websocket', 'Connection': 'Upgrade'},
    {'Upgrade': 'h2c', 'Connection': 'Upgrade, HTTP2-Settings'},
    {'Upgrade': 'webtransport', 'Connection': 'Upgrade'},
    {'Upgrade': 'TLS/1.3, HTTP/2', 'Connection': 'Upgrade'},
]
